# -*- coding: utf-8 -*-

"""
woocommerce
~~~~~~~~~~~~~~~
A Python wrapper for WooCommerce API.

:copyright: (c) 2019 by Automattic.
:license: MIT, see LICENSE for details.
"""

__title__ = "woocommerce"
__version__ = "3.0.0"
__author__ = "Claudio Sanches @ Automattic"
__license__ = "MIT"
